#ifndef __RATIONAL_H__
#define __RATIONAL_H__

/**
 * Most of my comments are in the implementation file Rational.cc
 * In C++ files are usually split up into header (or interface) files
 * and implementation files.  The header file gives the programming interface
 * of the class and decouples the implementation logic from its signature.
 */
#include <iostream.h>
#include "DivideByZeroException.h"

class Rational {

	// private, public, and protected fields are labels.  Everything
	// after the label, but before another label, has the visibility of 
	// that label.  hence _numerator and _denominator are private, the constructors etc. 
	// public, and the init and normalize functions again private
	private:
		int _numerator;
		int _denominator;

	public:

		// constructors		
		Rational();
		Rational(int numerator);
		Rational(int numerator, int denominator);
	
		// this is the copy constructor 
		Rational(const Rational& r);
		
		// The assignment operator	
		Rational& operator=(const Rational& r);
		
		Rational power(int power) const;
		
		inline int getNumerator() const;
		inline int getDenominator() const;
		
		const Rational& operator++();
		
		// this is the
		const Rational operator++(int);
		
		// friend means that operator+ can access the private and protected
		// member variables of any Rational object.  This must be declared
		// inside the class
		friend const Rational operator+(const Rational& r1, const Rational& r2);

		
	private:
		
		// We don't want the outside world using these.
		void init(int numerator, int denominator);
		void normalize();
		
};  // in C++, classes end with a semi-colon


// These functions don't live within the scope of a class.
// They are similar to static methods in Java, however, they are
// more like traditional functions in the C programming language

const Rational operator-(const Rational& r1, const Rational& r2);
const Rational operator*(const Rational& r1, const Rational& r2);

// By default in C++, all functions can throw any exception.
// When you declare that a function throws a particular exception you
// are restricting the Exceptions it can throw.  This is done mostly
// for back-compatability
const Rational operator/(const Rational& r1, const Rational& r2) throw (DivideByZeroException);
	
const bool operator==(const Rational& r1, const Rational& r2);
const bool operator<(const Rational& r1, const Rational& r2);
const bool operator<=(const Rational& r1, const Rational& r2);
const bool operator>(const Rational& r1, const Rational& r2);
const bool operator>=(const Rational& r1, const Rational& r2);


// This is the output stream operator 
ostream& operator<<(ostream& s, const Rational& r);	


#endif // __RATIONAL_H__